/**
 *
 * VkSimpleWindow.C
 *
 * This file contains the implementation for the VkSimpleWindow
 * class, which implements the base class for all windows that
 * appear on the screen (except dialogs, of course).
 *
 * Chris Toshok
 * Copyright (C) 1994
 * The Hungry Programmers, Inc
 *
 **/

#include <Vk/VkSimpleWindow.h>
#include <Vk/VkApp.h>
#include <iostream.h>
#include <stdlib.h>

#include <Xm/AtomMgr.h>
#include <Xm/Protocols.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#ifndef lint
static char rcsid[] = "$Id: VkSimpleWindow.C,v 1.10 1995/04/19 20:14:07 toshok Exp $";
#endif /* lint */

extern "C" {
  extern void _XEditResCheckMessages();
};

VkSimpleWindow::VkSimpleWindow(const char *name,
			       ArgList args,
			       Cardinal argCount) : VkComponent(name)
{
    Atom WM_DELETE_WINDOW;
    Atom WM_QUIT_APP;

    WM_DELETE_WINDOW = XmInternAtom(theApplication->display(),
				    "WM_DELETE_WINDOW", False);

    WM_QUIT_APP = XmInternAtom(theApplication->display(),
			       "_WM_QUIT_APP", True);

    _baseWidget = XtCreatePopupShell(name,
				     applicationShellWidgetClass,
				     theApplication->baseWidget(), args, argCount);
    
    XmAddWMProtocolCallback(_baseWidget, WM_DELETE_WINDOW, wm_delete_window, this);

    if (WM_QUIT_APP != None)
	XmAddWMProtocolCallback(_baseWidget, WM_QUIT_APP, wm_quit_app, this);

    XtAddEventHandler(_baseWidget, (EventMask)0, True, (XtEventHandler)_XEditResCheckMessages, NULL);

    installDestroyHandler();
  
    _mainWindowWidget = XmCreateMainWindow(_baseWidget,
					   (char *)name,
					   args,
					   argCount);

    XtManageChild(_mainWindowWidget);

    _windowTitle = strdup(name);
    _iconName = strdup(name);

    theApplication->registerWindow(this);
    _currentView = NULL;

    _iconState = ICON_UNKNOWN;
    _visibleState = VISIBLE_UNKNOWN;
    _stackingState = STACKING_UNKNOWN;
}

VkSimpleWindow::~VkSimpleWindow()
{
    free (_windowTitle);
    free (_iconName);
    theApplication->unregisterWindow(this);
    XtRemoveEventHandler(_baseWidget, StructureNotifyMask, False, 
			 change_state_handler, this);
}

const char *
VkSimpleWindow::className()
{
    return "VkSimpleWindow";
}

void
VkSimpleWindow::show()
{
    if (_visibleState == VISIBLE_UNKNOWN){
	/* This only happens when the window hasn't
	   appeared on the screen before. */

	if (theApplication->_startupIconified){
            Arg args[1];

            XtSetArg(args[0], XtNiconic, True);
            XtSetValues(_baseWidget, args, 1);

	    _iconState = CLOSED;
	}
	else
	    _iconState = OPEN;

	if (!_currentView)
	    addView(setUpInterface(mainWindowWidget()));

	XtPopup(_baseWidget, XtGrabNonexclusive);

	_visibleState = VISIBLE;

	XtAddEventHandler(_baseWidget, StructureNotifyMask, False, 
			  change_state_handler, this);

	setUpWindowProperties();
	afterRealizeHook();
    }

    else if (_visibleState == HIDDEN && _iconState != CLOSED){
	XtPopup(_baseWidget, XtGrabNonexclusive);
	_visibleState = VISIBLE;
    }
}

void
VkSimpleWindow::setUpWindowProperties()
{
    XTextProperty *wm_name = NULL;
    XTextProperty *icon_name = NULL; 

    if (_windowTitle){
	wm_name = (XTextProperty*)malloc(sizeof(XTextProperty));
	XStringListToTextProperty(&_windowTitle,
				  1,
				  wm_name);
    }

    if (_iconName){
	icon_name = (XTextProperty*)malloc(sizeof(XTextProperty));	
	XStringListToTextProperty(&_iconName,
				  1,
				  icon_name);
    }

    XSetWMProperties(theApplication->display(),
		     XtWindow(_baseWidget),
		     wm_name,
		     icon_name,
		     theApplication->argv(),
		     theApplication->argc(),
		     NULL, /* normal_hints */
		     NULL, /* wm_hints */
		     NULL  /* class_hints */);
  
    if (icon_name){
	XFree (icon_name->value);
	icon_name->value = NULL;
	free (icon_name);
    }

    if (wm_name){
	XFree (wm_name->value);
	wm_name->value = NULL;
	free (wm_name);
    }
}

void
VkSimpleWindow::hide()
{
    XtPopdown(_baseWidget);
    _visibleState = HIDDEN;
}

void
VkSimpleWindow::iconify()
{
    if (_visibleState != HIDDEN && _iconState != CLOSED){
	XIconifyWindow(theApplication->display(), 
		       XtWindow(_baseWidget),
		       XScreenNumberOfScreen(XtScreen(_baseWidget)));
	
	stateChanged(CLOSED);
    }
}

void
VkSimpleWindow::open()
{
    if (_iconState == CLOSED){
	XtPopup(_baseWidget, XtGrabNonexclusive);
	stateChanged(OPEN);
    }
}

void
VkSimpleWindow::raise()
{
//    XRaiseWindow(theApplication->display(),
//		 XtWindow(baseWidget()));
}

void
VkSimpleWindow::lower()
{
    XLowerWindow(theApplication->display(),
		 XtWindow(baseWidget()));
}

void
VkSimpleWindow::handleRawEvent(XEvent *event)
{
    /* empty */
}

Widget
VkSimpleWindow::mainWindowWidget() const
{
    return _mainWindowWidget;
}

void
VkSimpleWindow::addView(Widget w)
{
    if (_currentView)
	removeView();
    XtManageChild(w);
    _currentView = w;
}

void
VkSimpleWindow::addView(VkComponent *component)
{
    if (_currentView)
	removeView();
    XtManageChild(*component);
    _currentView = *component;
}

void
VkSimpleWindow::removeView()
{
    XtUnmanageChild(_currentView);
    _currentView = NULL;
}

Widget
VkSimpleWindow::viewWidget() const
{
    return _currentView;
}

Boolean
VkSimpleWindow::visible() const
{
    return _visibleState == VISIBLE;
}

Boolean
VkSimpleWindow::iconic() const
{
    return _iconState == CLOSED;
}

void
VkSimpleWindow::stateChanged(IconState newState)
{
    _iconState = newState;
}

void
VkSimpleWindow::afterRealizeHook()
{
    //  do nothing.  let the user override.
}

void
VkSimpleWindow::setTitle(const char *newTitle)
{
    XTextProperty wm_name;
    // for now, we just use the uniterpreted string --- We don't check if it's a resource
    delete _windowTitle;
    _windowTitle = strdup(newTitle);
 
    XStringListToTextProperty(&_windowTitle, 1, &wm_name);

    XSetWMName(theApplication->display(),
	       XtWindow(_baseWidget),
	       &wm_name);
}

const char*
VkSimpleWindow::getTitle()
{
    return _windowTitle;
}

void
VkSimpleWindow::setIconName(const char *newTitle)
{
    XTextProperty icon_name;

    // again, we don't check right now whether it's a resource of not...
    delete _iconName;
    _iconName = strdup(newTitle);

    XStringListToTextProperty(&_iconName, 1, &icon_name);

    XSetWMIconName(theApplication->display(),
		   XtWindow(_baseWidget),
		   &icon_name);
}

void
VkSimpleWindow::setIconPixmap(Pixmap pix)
{
    Arg args[1];

    XtSetArg(args[0], XtNiconPixmap, pix);

    XtSetValues(_baseWidget, args, 1);
}

void
VkSimpleWindow::handleWMDeleteMessage()
{
    delete this;
}

void
VkSimpleWindow::handleWMQuitMessage()
{
    theApplication->quitYourself();
}

void
VkSimpleWindow::wm_delete_window(Widget w, XtPointer clientData, XtPointer)
{
    VkSimpleWindow *obj = (VkSimpleWindow *)clientData;

    obj->handleWMDeleteMessage();
}

void
VkSimpleWindow::wm_quit_app(Widget, XtPointer clientData, XtPointer)
{
    VkSimpleWindow *obj = (VkSimpleWindow*)clientData;

    obj->handleWMQuitMessage();
}

void
VkSimpleWindow::change_state_handler(Widget,
				     XtPointer clientData,
				     XEvent *event,
				     Boolean *)
{
    VkSimpleWindow *obj = (VkSimpleWindow*)clientData;

    obj->changeState(event);
}

void
VkSimpleWindow::changeState(XEvent *event)
{
    XCirculateEvent *xcirculate;
    XConfigureEvent *xconfigure;

    switch (event->type){
    case CirculateNotify:
	xcirculate = (XCirculateEvent*)event;
	if (xcirculate->place == PlaceOnTop)
	    _stackingState = RAISED;
	else
	    _stackingState = LOWERED;
	break;
    case ConfigureNotify:
	xconfigure = (XConfigureEvent*)event;
	if (xconfigure->above == None)
	    _stackingState = LOWERED;
	else
	    _stackingState = STACKING_UNKNOWN;
	break;
    }
}
